#!/usr/bin/python
import os
import re
from setuptools import setup
from m2r import parse_from_file
import restructuredtext_lint

# Parser README.md into reStructuredText format
rst_readme = parse_from_file('README.md')

# Validate the README, checking for errors
errors = restructuredtext_lint.lint(rst_readme)

# Raise an exception for any errors found
if errors:
    print(rst_readme)
    raise ValueError('README.md contains errors: ',
                     ', '.join([e.message for e in errors]))

# Attempt to get version number from TravisCI environment variable
version = os.environ.get('TRAVIS_TAG', default='0.0.0')

# Remove leading 'v'
version = re.sub('^v', '', version)

setup(
    name='auto-all',
    description='Automatically manage __all__ variable in Python packages.',
    long_description=rst_readme,
    version=version,
    author='Jon Grace-Cox',
    author_email='jongracecox@gmail.com',
    py_modules=['auto_all'],
    setup_requires=['setuptools', 'wheel'],
    tests_require=['unittest'],
    install_requires=[],
    data_files=[],
    options={
        'bdist_wheel': {'universal': True}
    },
    url='https://github.com/jongracecox/auto-all',
    classifiers=[
        'License :: OSI Approved :: MIT License'
    ]
)



"""
Auto-All package

auto_all can be used for controlling what is made available
for import from a package.

Example usage:

    First, import the auto_all functions.

        >>> from auto_all import start_all, end_all

    If your package has external dependencies then these can be imported
    and the imported objects can be hidden.  In this case we will import
    pathlib.Path and show that it doesn't appear on the __all__ list.

        >>> from pathlib import Path

    Now we can define some internal functions that we want to keep private.
    We can also do this using underscore prefixes, but auto_all gives us a
    little more control.

        >>> def a_private_function():
        ...     print("This is a private function.")

    Now we are ready to start defining public functions, so we use
    ``start_all()``.  We need to pass it the globals dict so that it can
    see what's already defined.

        >>> start_all(globals())

    Now we can define our public functions.

        >>> def a_public_function():
        ...     print("This is a public function.")

    Finally we use ``end_all()`` to finish defining public functions and
    create the ``__all__`` variable.

        >>> end_all(globals())

    When we look at the ``__all__`` variable we can see only the public
    facing function is listed.

        >>> print(__all__)
        ['a_public_function']

"""
from typing import Dict
_GLOBAL_VAR_NAME = '_do_not_include_all'


def start_all(globs: Dict):
    """Start defining externally accessible objects.

    Call ``start_all(globals())`` when you want to start defining objects
    in your module that you want to be accessible from outside the module.

    Args:
        globs(dict): Pass the globals dictionary to the function using
            ``globals()``.
    """
    globs[_GLOBAL_VAR_NAME] = list(globs.keys()) + [_GLOBAL_VAR_NAME]


def end_all(globs: Dict):
    """Finish defining externally accessible objects.

    Call ``end_all(globals())`` when you have finished defining objects
    in your module that you want to be accessible from outside the module.
    After this function has been run the ``__all__`` module variable will
    be updated with the names of all objects that were created between the
    ``start_all`` and ``end_all`` funciton calls.

    Args:
        globs(dict): Pass the globals dictionary to the function using
            ``globals()``.
    """
    globs['__all__'] = list(
        set(list(globs.keys())) - set(globs[_GLOBAL_VAR_NAME])
    )


__all__ = ['start_all', 'end_all']
